@@ -13,16 +13,15 @@
 # limitations under the License.
 import inspect
 import os
-from argparse import ArgumentParser, Namespace
+from argparse import _ArgumentGroup, ArgumentParser, Namespace
 from contextlib import suppress
 from typing import Any, Dict, List, Tuple, Union
 
-from pytorch_lightning.utilities import parsing
+from pytorch_lightning.utilities.parsing import str_to_bool, str_to_bool_or_int, str_to_bool_or_str
 
 
 def from_argparse_args(cls, args: Union[Namespace, ArgumentParser], **kwargs):
-    """
-    Create an instance from CLI arguments.
+    """Create an instance from CLI arguments.
     Eventually use varibles from OS environement which are defined as "PL_<CLASS-NAME>_<CLASS_ARUMENT_NAME>"
 
     Args:
@@ -68,7 +67,7 @@ def parse_argparser(cls, arg_parser: Union[ArgumentParser, Namespace]) -> Namesp
                 # Value has been passed as a flag => It is currently None, so we need to set it to True
                 # We always set to True, regardless of the default value.
                 # Users must pass False directly, but when passing nothing True is assumed.
-                # i.e. the only way to disable somthing that defaults to True is to use the long form:
+                # i.e. the only way to disable something that defaults to True is to use the long form:
                 # "--a_default_true_arg False" becomes False, while "--a_default_false_arg" becomes None,
                 # which then becomes True here.
 
@@ -135,37 +134,84 @@ def get_init_arguments_and_types(cls) -> List[Tuple[str, Tuple, Any]]:
     return name_type_default
 
 
-def add_argparse_args(cls, parent_parser: ArgumentParser) -> ArgumentParser:
-    r"""Extends existing argparse by default `Trainer` attributes.
+def _get_abbrev_qualified_cls_name(cls):
+    assert isinstance(cls, type), repr(cls)
+    if cls.__module__.startswith("pytorch_lightning."):
+        # Abbreviate.
+        return f"pl.{cls.__name__}"
+    else:
+        # Fully qualified.
+        return f"{cls.__module__}.{cls.__qualname__}"
+
+
+def add_argparse_args(
+    cls,
+    parent_parser: ArgumentParser,
+    *,
+    use_argument_group=True,
+) -> ArgumentParser:
+    r"""Extends existing argparse by default attributes for ``cls``.
 
     Args:
         cls: Lightning class
         parent_parser:
             The custom cli arguments parser, which will be extended by
-            the Trainer default arguments.
+            the class's default arguments.
+        use_argument_group:
+            By default, this is True, and uses ``add_argument_group`` to add
+            a new group.
+            If False, this will use old behavior.
+
+    Returns:
+        If use_argument_group is True, returns ``parent_parser`` to keep old
+        workflows. If False, will return the new ArgumentParser object.
 
     Only arguments of the allowed types (str, float, int, bool) will
-    extend the `parent_parser`.
+    extend the ``parent_parser``.
 
     Examples:
+
+        # Option 1: Default usage.
         >>> import argparse
         >>> from pytorch_lightning import Trainer
         >>> parser = argparse.ArgumentParser()
         >>> parser = Trainer.add_argparse_args(parser)
         >>> args = parser.parse_args([])
-    """
-    parser = ArgumentParser(
-        parents=[parent_parser],
-        add_help=False,
-    )
 
-    blacklist = ['kwargs']
-    depr_arg_names = cls.get_deprecated_arg_names() + blacklist
+        # Option 2: Disable use_argument_group (old behavior).
+        >>> import argparse
+        >>> from pytorch_lightning import Trainer
+        >>> parser = argparse.ArgumentParser()
+        >>> parser = Trainer.add_argparse_args(parser, use_argument_group=False)
+        >>> args = parser.parse_args([])
+    """
+    if isinstance(parent_parser, _ArgumentGroup):
+        raise RuntimeError("Please only pass an ArgumentParser instance.")
+    if use_argument_group:
+        group_name = _get_abbrev_qualified_cls_name(cls)
+        parser = parent_parser.add_argument_group(group_name)
+    else:
+        parser = ArgumentParser(
+            parents=[parent_parser],
+            add_help=False,
+        )
+
+    ignore_arg_names = ['self', 'args', 'kwargs']
+    if hasattr(cls, "get_deprecated_arg_names"):
+        ignore_arg_names += cls.get_deprecated_arg_names()
 
     allowed_types = (str, int, float, bool)
 
-    args_help = parse_args_from_docstring(cls.__init__.__doc__ or cls.__doc__)
-    for arg, arg_types, arg_default in (at for at in get_init_arguments_and_types(cls) if at[0] not in depr_arg_names):
+    # Get symbols from cls or init function.
+    for symbol in (cls, cls.__init__):
+        args_and_types = get_init_arguments_and_types(symbol)
+        args_and_types = [x for x in args_and_types if x[0] not in ignore_arg_names]
+        if len(args_and_types) > 0:
+            break
+
+    args_help = _parse_args_from_docstring(cls.__init__.__doc__ or cls.__doc__ or "")
+
+    for arg, arg_types, arg_default in args_and_types:
         arg_types = [at for at in allowed_types if at in arg_types]
         if not arg_types:
             # skip argument with not supported type
@@ -175,9 +221,11 @@ def add_argparse_args(cls, parent_parser: ArgumentParser) -> ArgumentParser:
             arg_kwargs.update(nargs="?", const=True)
             # if the only arg type is bool
             if len(arg_types) == 1:
-                use_type = parsing.str_to_bool
+                use_type = str_to_bool
+            elif int in arg_types:
+                use_type = str_to_bool_or_int
             elif str in arg_types:
-                use_type = parsing.str_to_bool_or_str
+                use_type = str_to_bool_or_str
             else:
                 # filter out the bool as we need to use more general
                 use_type = [at for at in arg_types if at is not bool][0]
@@ -204,10 +252,13 @@ def add_argparse_args(cls, parent_parser: ArgumentParser) -> ArgumentParser:
             **arg_kwargs,
         )
 
-    return parser
+    if use_argument_group:
+        return parent_parser
+    else:
+        return parser
 
 
-def parse_args_from_docstring(docstring: str) -> Dict[str, str]:
+def _parse_args_from_docstring(docstring: str) -> Dict[str, str]:
     arg_block_indent = None
     current_arg = None
     parsed = {}
@@ -237,6 +288,13 @@ def _gpus_allowed_type(x) -> Union[int, str]:
         return int(x)
 
 
+def _gpus_arg_default(x) -> Union[int, str]:  # pragma: no-cover
+    # unused, but here for backward compatibility with old checkpoints that need to be able to
+    # unpickle the function from the checkpoint, as it was not filtered out in versions < 1.2.8
+    # see: https://github.com/PyTorchLightning/pytorch-lightning/pull/6898
+    pass
+
+
 def _int_or_float_type(x) -> Union[int, float]:
     if '.' in str(x):
         return float(x)
